package com.gmail.romanesco2090.base;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Deque;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.aiwolf.client.lib.Content;
import org.aiwolf.client.lib.Topic;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Player;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Talk;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

import com.gmail.romanesco2090.TalkFactory;
import com.gmail.romanesco2090.data.AgentData;
import com.gmail.romanesco2090.data.GameResult;
import com.gmail.romanesco2090.util.Debug;

public class BaseRole implements Player {

	/** ゲーム結果 */
	protected GameResult gameResult;

	/** ゲーム情報 */
	protected GameInfo gameInfo;

	/** finish通過フラグ */
	protected boolean isPassFinish;

	/** 5人人狼かどうか */
	protected boolean isFive;

	/** エージェント情報管理マップ */
	public Map<Agent, AgentData> AgentDataMap;

	/** 1日ごとの発言された回数 */
	public int readTalkNum;

	/** ターン数 */
	protected int turn;

	/** whisperのターン数 */
	protected int whisperturn;

	/** 自分が話す内容のQue */
	protected Deque<String> myTalkQue;

	/** 自分の呻き内容のQue */
	protected Deque<String> myWhisperQue;

	/** 自分が投票する候補者 */
	protected List<Agent> votingCandidate;

	/** その日の投票対象エージェント */
	protected Agent votingTarget;

	/** 自分以外のカミングアウトしたエージェントのマップ */
	protected Map<Agent, Role> comingoutMap;

	/** エージェントそれぞれの発話のMap */
	protected Map<Agent, List<Talk>> talkMap;

	/** エージェントごとの被投票による評価値 */
	protected Map<Agent, Double> votedScoreMap;

	/** エージェントの発話回数 */
	protected Map<Agent, Integer> voteCountMap;

	public BaseRole(GameResult gameResult) {
		this.gameResult = gameResult;
	}

	@Override
	public void initialize(GameInfo arg0, GameSetting arg1) {
		try {
			gameInfo = arg0;
			isPassFinish = false;
			gameResult.init(arg0);
			AgentDataMap = new HashMap<>();
			readTalkNum = 0;
			turn = 0;
			myTalkQue = new LinkedList<>();
			myWhisperQue = new LinkedList<>();
			comingoutMap = new HashMap<>();
			votingCandidate = new LinkedList<>();
			votingTarget = null;
			talkMap = new HashMap<>();
			votedScoreMap = new HashMap<>();
			voteCountMap = new HashMap<>();
			for (Agent agent : gameInfo.getAgentList()) {
				voteCountMap.put(agent, 1);
			}
			//talkMap初期化
			for (Agent agent : gameInfo.getAgentList()) {
				talkMap.put(agent, new ArrayList<>());
			}
			if (arg1.getPlayerNum() <= 5) {
				isFive = true;
			} else {
				isFive = false;
			}
			//エージェント情報初期化
			for (Agent agent : gameInfo.getAgentList()) {
				AgentDataMap.put(agent, new AgentData(gameInfo, agent));
			}

			if (isFive) {
				initializeAction();
			} else {
				initializeAction15();
			}

		} catch (Exception e) {
			this.writeEx(e);
		}
	}

	@Override
	public void update(GameInfo arg0) {
		try {
			gameInfo = arg0;

			List<Talk> talkList = gameInfo.getTalkList();
			for (int i = readTalkNum; i < talkList.size(); i++) {
				Talk talk = talkList.get(i);
				Agent talker = talk.getAgent();
				Content content = new Content(talk.getText());
				//自分の発言には反応しない
				if (talker == gameInfo.getAgent()) {
					continue;
				}
				//CO発言された時の処理
				if (content.getTopic() == Topic.COMINGOUT) {
					comingoutMap.put(talker, content.getRole());
				}
				//投票発言した時の処理
				if (content.getTopic() == Topic.VOTE) {
					//isVoteMap.put(talker, true);
					voteCountMap.put(talker, voteCountMap.get(talker) + 1);
				}

				//talkMap更新
				List<Talk> storeList = new ArrayList<>();
				if (talkMap.containsKey(talker)) {
					storeList = talkMap.get(talker);
				}
				storeList.add(talk);
				talkMap.put(talker, storeList);

			}

			if (isFive) {
				updateAction();
			} else {
				updateAction15();
			}

			readTalkNum++;

		} catch (Exception e) {
			this.writeEx(e);
		}
	}

	@Override
	public void dayStart() {
		try {
			readTalkNum = 0;
			whisperturn = 0;
			turn = 0;
			whisperturn = 0;
			votingTarget = null;
			votedScoreMap.clear();
			if (isFive) {
				dayStartAction();
			} else {
				dayStartAction15();
			}

		} catch (Exception e) {
			this.writeEx(e);
		}
	}

	@Override
	public String talk() {
		try {
			if (isFive) {
				talkAction();
			} else {
				talkAction15();
			}
		} catch (Exception e) {
			this.writeEx(e);
		}
		turn++;
		if (!myTalkQue.isEmpty()) {
			return myTalkQue.poll();
		}
		return TalkFactory.skipRemark();
	}

	@Override
	public Agent vote() {
		try {

			if (isFive) {
				return voteAction();
			} else {
				return voteAction15();
			}

		} catch (Exception e) {
			this.writeEx(e);
		}
		return null;
	}

	@Override
	public Agent divine() {
		try {
			Agent Target = null;
			if (isFive) {
				Target = divineAction();
				return Target;
			} else {
				Target = divineAction15();
				return Target;
			}

		} catch (Exception e) {
			this.writeEx(e);
		}
		return null;
	}

	@Override
	public String whisper() {
		try {

			if (isFive) {
				//whisperAction();
			} else {
				whisperAction15();
			}

		} catch (Exception e) {
			this.writeEx(e);
		}
		whisperturn++;
		if (!myWhisperQue.isEmpty()) {
			return myWhisperQue.poll();
		}
		return TalkFactory.skipRemark();
	}

	@Override
	public Agent guard() {
		try {

			return guardAction();

		} catch (Exception e) {
			this.writeEx(e);
		}
		return null;
	}

	@Override
	public Agent attack() {
		try {

			if (isFive) {
				return attackAction();
			} else {
				return attackAction15();
			}

		} catch (Exception e) {
			this.writeEx(e);
		}
		return null;
	}

	@Override
	public void finish() {
		try {
			//finish通過確認
			if (!isPassFinish) {
				isPassFinish = true;
				return;
			}
			gameResult.updateData(gameInfo);
			if (isFive) {
				finishAction();
			} else {
				finishAction15();
			}
		} catch (Exception e) {
			this.writeEx(e);
		}
		Debug.showError();
	}

	@Override
	public String getName() {
		try {

		} catch (Exception e) {
			this.writeEx(e);
		}
		return null;
	}

	/**
	 * 5人人狼のときinitializeで行う処理
	 */
	public void initializeAction() {
	}

	/**
	 * 15人人狼のときinitializeで行う処理
	 */
	public void initializeAction15() {
	}

	/**
	 * 5人人狼のときupdateで行う処理
	 */
	public void updateAction() {
	}

	/**
	 * 15人人狼のときupdateで行う処理
	 */
	public void updateAction15() {
	}

	/**
	 * 5人人狼のときdayStartで行う処理
	 */
	public void dayStartAction() {
	}

	/**
	 * 15人人狼のときdayStartで行う処理
	 */
	public void dayStartAction15() {
	}

	/**
	 * 5人人狼のときtalkで行う処理
	 */
	public void talkAction() {
	}

	/**
	 * 15人人狼のときtalkで行う処理
	 */
	public void talkAction15() {
	}

	/**
	 * 5人人狼のときvoteで行う処理
	 */
	public Agent voteAction() {
		return null;
	}

	/**
	 * 15人人狼のときvoteで行う処理
	 */
	public Agent voteAction15() {
		return null;
	}

	/**
	 * 5人人狼のときdevineで行う処理
	 */
	public Agent divineAction() {
		return null;
	}

	/**
	 * 15人人狼のときdevineで行う処理
	 */
	public Agent divineAction15() {
		return null;
	}

	/**
	 * 15人人狼のときwhisperで行う処理
	 */
	public void whisperAction15() {
	}

	/**
	 * 15人人狼のときguardで行う処理
	 */
	public Agent guardAction() {
		return null;
	}

	/**
	 * 5人人狼のときattackで行う処理
	 */
	public Agent attackAction() {
		return null;
	}

	/**
	 * 15人人狼のときattackで行う処理
	 */
	public Agent attackAction15() {
		return null;
	}

	/**
	 * 5人人狼のときfinishで行う処理
	 */
	public void finishAction() {
	}

	/**
	 * 15人人狼のときfinishで行う処理
	 */
	public void finishAction15() {
	}

	/**
	 * 自分以外の生存しているエージェントリストを返すメソッド
	 * @return 生存しているエージェントリスト
	 */
	public List<Agent> getAliveOthersList() {
		List<Agent> aliveAgentList = new LinkedList<>();
		aliveAgentList = gameInfo.getAliveAgentList();
		aliveAgentList.remove(gameInfo.getAgent());
		return aliveAgentList;
	}


	/**
	 * 評価値 = (対象に投票発言したターンの合計) / (投票発言数)
	 * @param talkMap
	 * @param voteCountMap
	 * @return
	 */
	public Map<Agent, Double> calculateVotedScore(Map<Agent, List<Talk>> talkMap, Map<Agent, Integer> voteCountMap) {
		Map<Agent, Double> votedScoreMap = new HashMap<>();

		for (Agent talker : gameInfo.getAliveAgentList()) {
			if (talker == gameInfo.getAgent()) {
				continue;
			}
			for (Talk talk : talkMap.get(talker)) {
				Content content = new Content(talk.getText());
				Agent target = content.getTarget();
				if (content.getTopic() == Topic.VOTE) {
					if (!votedScoreMap.containsKey(target)) {
						votedScoreMap.put(target, 0.0);
					}
					votedScoreMap.put(target,
							votedScoreMap.get(target) + ((double) talk.getTurn() / (double) voteCountMap.get(talker)));
				}
			}
		}
		return votedScoreMap;
	}

	/**
	 * votedScoreMapから投票先の候補者リストを返すメソッド
	 * それぞれのエージェントに対して評価値を与える
	 * 評価値 = (投票発言のターンの合計) / (投票発言数)
	 * @param scoreMap
	 * @return 最も評価値の高いエージェントリスト
	 */
	public List<Agent> decideCnaditate(Map<Agent, Double> scoreMap) {
		double maxValue = 0;
		List<Agent> cd = new LinkedList<>();
		for (Agent agent : gameInfo.getAliveAgentList()) {
			if (!scoreMap.containsKey(agent)) {
				continue;
			}
			if (scoreMap.get(agent) == maxValue) {
				cd.add(agent);
			} else if (scoreMap.get(agent) > maxValue) {
				maxValue = scoreMap.get(agent);
				cd.clear();
				cd.add(agent);
			}
		}
		return cd;
	}

	/**
	 * エラー文記述
	 * @param e エラー
	 */
	public void writeEx(Exception e) {
		StringWriter stringWriter = new StringWriter();
		PrintWriter printWriter = new PrintWriter(stringWriter);
		e.printStackTrace(printWriter);
		Debug.stackError(stringWriter);
	}

}
