package com.gmail.romanesco2090.data;

import java.util.HashMap;
import java.util.Map;

import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.net.GameInfo;

import com.gmail.romanesco2090.util.Debug;

public class GameResult {

	/** ゲーム回数 */
	protected int gameCount;
	/** 勝利回数マップ */
	public Map<Agent, Integer> winMap;
	/** 初期化するかどうかのフラグ */
	protected boolean isInit = false;

	/**
	 * 初期化処理
	 * @param gameInfo ゲーム情報
	 */
	public void init(GameInfo gameInfo) {
		if(isInit) {
			return;
		}
		gameCount = 0;
		winMap = new HashMap<>();
		gameInfo.getAgentList().stream().forEach(k -> winMap.put(k, 0));
		isInit = true;
	}

	/**
	 * ゲーム回数取得
	 * @return
	 */
	public int getGameCount() {
		return this.gameCount;
	}

	/**
	 * ゲーム結果更新処理
	 * @param gameInfo ゲーム情報
	 */
	public void updateData(GameInfo gameInfo) {
		gameCount++;
		this.winCount(gameInfo);
	}

	/**
	 * エージェントごとに勝利回数をカウントする
	 * @param gameInfo ゲーム情報
	 */
	public void winCount(GameInfo gameInfo) {
		//人狼陣営の勝利フラグ
		boolean winWolf = false;

		//勝利判定
		for(Agent agent: gameInfo.getAliveAgentList()) {
			Role role = gameInfo.getRoleMap().get(agent);
			if(role == Role.WEREWOLF) {
				winWolf = true;
				break;
			}
		}

		//エージェント勝利回数カウント
		for(Agent agent: gameInfo.getAgentList()) {
			Role role = gameInfo.getRoleMap().get(agent);
			if(winWolf) {
				if(role == Role.WEREWOLF || role == Role.POSSESSED) {
					winMap.put(agent, winMap.get(agent)+1);
				}
			} else {
				if(role != Role.WEREWOLF && role != Role.POSSESSED) {
					winMap.put(agent, winMap.get(agent)+1);
				}
			}
		}
	}

	/**
	 * winMap取得
	 * @return
	 */
	public Map<Agent, Integer> getWinMap() {
		return winMap;
	}

	/**
	 * winMap表示
	 */
	public void show() {
		Debug.error(winMap);
		Debug.error(gameCount);
	}
}
