package com.gmail.romanesco2090.role;

import java.util.LinkedList;
import java.util.List;

import org.aiwolf.client.lib.Content;
import org.aiwolf.client.lib.Topic;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.data.Talk;

import com.gmail.romanesco2090.TalkFactory;
import com.gmail.romanesco2090.base.BaseRole;
import com.gmail.romanesco2090.data.GameResult;
import com.gmail.romanesco2090.util.Gadget;
import com.gmail.romanesco2090.util.RandomSelect;

public class Werewolf extends BaseRole {

	public Werewolf(GameResult gameResult) {
		super(gameResult);
	}

	/** 黒出しされたかどうか */
	protected boolean blackFlg;

	/** 自分が襲撃する候補者 */
	protected List<Agent> attackingCandidate;

	/** その日の襲撃対象エージェント */
	protected Agent attackingTarget;

	/** 再投票フラグ */
	protected boolean revoteFlg;

	/** 占い師戦略するかどうか */
	protected boolean seerTac;

	/** 人狼仲間 */
	List<Agent> friends;

	@Override
	public void initializeAction() {
		blackFlg = false;
		attackingCandidate = new LinkedList<>();
		attackingTarget = null;
		revoteFlg = false;
		seerTac = true;
		if (RandomSelect.getNum(5) == 0) {
			seerTac = false;
		}
	}

	@Override
	public void updateAction() {
		//1日目
		if (gameInfo.getDay() == 1) {
			List<Talk> talkList = gameInfo.getTalkList();
			for (int i = readTalkNum; i < talkList.size(); i++) {
				Talk talk = talkList.get(i);
				Agent talker = talk.getAgent();
				Content content = new Content(talk.getText());
				//自分の発言には反応しない
				if (talker == gameInfo.getAgent()) {
					continue;
				}
				//占い結果発言時の処理
				if (content.getTopic() == Topic.DIVINED) {
					if (comingoutMap.containsKey(talker)) {
						if (comingoutMap.get(talker) == Role.SEER) {
							//白の時
							if (content.getResult() == Species.HUMAN) {
								if (votingCandidate.contains(content.getTarget())) {
									votingCandidate.remove(content.getTarget());
								}
							}
							//黒の時
							else {
								blackFlg = true;
								if (content.getTarget() != gameInfo.getAgent()
										&& !votingCandidate.contains(content.getTarget())) {
									votingCandidate.add(content.getTarget());
								}
							}
						}
					}
				}
			}
		}
		//2日目
		else {

		}
	}

	@Override
	public void dayStartAction() {
		attackingTarget = null;
		revoteFlg = false;
	}

	@Override
	public void talkAction() {
		//1日目
		if (gameInfo.getDay() == 1) {
			if (seerTac) {
				if (turn == 0) {
					myTalkQue.offer(TalkFactory.comingoutRemark(gameInfo.getAgent(), Role.SEER));
				} else if (turn == 1) {
					List<Agent> seerList = new LinkedList<>();
					for (Agent agent : comingoutMap.keySet()) {
						if (comingoutMap.get(agent) == Role.SEER) {
							seerList.add(agent);
						}
					}
					votingCandidate = getAliveOthersList();
					Gadget.remainList(votingCandidate, seerList);
					votingTarget = RandomSelect.getOP(gameResult.winMap, seerList);
					myTalkQue.offer(TalkFactory.divinedResultRemark(votingTarget, Species.WEREWOLF));
					myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
				}
			} else {
				//黒出し発言されたら発言する
				if (blackFlg) {
					votingTarget = RandomSelect.getOP(gameResult.getWinMap(), votingCandidate);
					myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
					blackFlg = false;
				} else if (turn == 2) {
					List<Agent> seerList = new LinkedList<>();
					for (Agent agent : comingoutMap.keySet()) {
						if (comingoutMap.get(agent) == Role.SEER) {
							seerList.add(agent);
						}
					}
					votingCandidate = getAliveOthersList();
					Gadget.remainList(votingCandidate, seerList);
					votingTarget = RandomSelect.getOP(gameResult.winMap, seerList);
					myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
				}
			}
		}
		//2日目
		else {
			if (turn == 0) {
				myTalkQue.offer(TalkFactory.comingoutRemark(gameInfo.getAgent(), Role.WEREWOLF));
				votingTarget = RandomSelect.getOP(gameResult.getWinMap(), getAliveOthersList());
				myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
			}
		}
	}

	@Override
	public Agent voteAction() {
		if (revoteFlg) {
			votingTarget = RandomSelect.get(Gadget.remainList(getAliveOthersList(), votingTarget));
		}
		revoteFlg = true;
		if (votingTarget != null) {
			return votingTarget;
		} else if (!votingCandidate.isEmpty()) {
			return RandomSelect.getOP(gameResult.getWinMap(), votingCandidate);
		} else {
			return RandomSelect.getOP(gameResult.getWinMap(), getAliveOthersList());
		}
	}

	@Override
	public Agent attackAction() {
		//1日目は占い師CO以外を襲撃
		if (gameInfo.getDay() == 1) {
			attackingCandidate = getAliveOthersList();
			for (Agent agent : getAliveOthersList()) {
				if (comingoutMap.containsKey(agent)) {
					if (comingoutMap.get(agent) == Role.SEER) {
						attackingCandidate.remove(agent);
					}
				}
			}
		}
		return RandomSelect.getOP(gameResult.getWinMap(), attackingCandidate);

	}

	@Override
	public void initializeAction15() {
		blackFlg = false;
		friends = new LinkedList<>();
		for (Agent agent : gameInfo.getRoleMap().keySet()) {
			if (gameInfo.getRoleMap().get(agent) == Role.WEREWOLF) {
				friends.add(agent);
			}
		}
	}

	@Override
	public void updateAction15() {
		List<Talk> talkList = gameInfo.getTalkList();
		for (int i = readTalkNum; i < talkList.size(); i++) {
			Talk talk = talkList.get(i);
			Agent talker = talk.getAgent();
			Content content = new Content(talk.getText());
			//自分の発言には反応しない
			if (talker == gameInfo.getAgent()) {
				continue;
			}
			//CO発言された時の処理
			if (content.getTopic() == Topic.DIVINED) {
				if (content.getResult() == Species.WEREWOLF) {
					if (!friends.contains(content.getTarget())) {
						blackFlg = true;
					}
				}
			}
		}

	}

	@Override
	public void dayStartAction15() {
	}

	@Override
	public void talkAction15() {
//		if (gameInfo.getDay() == 1) {
//			if (turn == 0) {
//				myTalkQue.offer(TalkFactory.comingoutRemark(gameInfo.getAgent(), Role.MEDIUM));
//			}
//			if (turn == 3 || turn == 5 || turn == 7) {
//				votingCandidate = decideCnaditate(calculateVotedScore(talkMap, voteCountMap));
//				votingTarget = RandomSelect.getOP(gameResult.winMap, votingCandidate);
//				myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
//			}
//
//		} else if (gameInfo.getDay() <= 4) {
//			if (turn == 0) {
//				//狂人にあわせる それ以外は真実
//				if (blackFlg) {
//					myTalkQue.offer(TalkFactory.identRemark(gameInfo.getExecutedAgent(), Species.WEREWOLF));
//					blackFlg = false;
//				} else {
//					if (friends.contains(gameInfo.getExecutedAgent())) {
//						myTalkQue.offer(TalkFactory.identRemark(gameInfo.getExecutedAgent(), Species.WEREWOLF));
//					} else {
//						myTalkQue.offer(TalkFactory.identRemark(gameInfo.getExecutedAgent(), Species.HUMAN));
//					}
//				}
//			}
//			if (turn == 3 || turn == 5 || turn == 7) {
//				votingCandidate = decideCnaditate(calculateVotedScore(talkMap, voteCountMap));
//				votingTarget = RandomSelect.getOP(gameResult.winMap, votingCandidate);
//				myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
//			}
//		} else {
//			if (turn == 0) {
//				//狂人にあわせる それ以外は真実
//				if (blackFlg) {
//					myTalkQue.offer(TalkFactory.identRemark(gameInfo.getExecutedAgent(), Species.WEREWOLF));
//					blackFlg = false;
//				} else {
//					if (friends.contains(gameInfo.getExecutedAgent())) {
//						myTalkQue.offer(TalkFactory.identRemark(gameInfo.getExecutedAgent(), Species.WEREWOLF));
//					} else {
//						myTalkQue.offer(TalkFactory.identRemark(gameInfo.getExecutedAgent(), Species.HUMAN));
//					}
//				}
//			}
			if (turn == 3 || turn == 5 || turn == 7) {
				votingCandidate = decideCnaditate(calculateVotedScore(talkMap, voteCountMap));
				votingTarget = RandomSelect.getOP(gameResult.winMap, votingCandidate);
				//5ターン目以降は人狼に投票しない
				if (gameInfo.getDay() >= 5 && friends.contains(votingTarget)) {
					votingCandidate = Gadget.remainList(getAliveOthersList(), friends);
					votingTarget = RandomSelect.getOP(gameResult.winMap, votingCandidate);
					myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
				} else {
					myTalkQue.offer(TalkFactory.voteRemark(votingTarget));
				}
			}
		}
//	}

	@Override
	public Agent voteAction15() {
		if (votingTarget != null) {
			return votingTarget;
		} else if (!votingCandidate.isEmpty()) {
			return RandomSelect.getOP(gameResult.getWinMap(), votingCandidate);
		} else {
			return RandomSelect.getOP(gameResult.getWinMap(), getAliveOthersList());
		}
	}

	@Override
	public void whisperAction15() {
		if (gameInfo.getDay() == 0 && whisperturn == 0) {
			//			myWhisperQue.offer(TalkFactory.comingoutRemark(gameInfo.getAgent(), Role.MEDIUM));
		} else if (whisperturn == 0) {
			attackingCandidate = getAliveOthersList();
			Gadget.remainList(attackingCandidate, friends);
			attackingTarget = RandomSelect.getOP(gameResult.winMap, attackingCandidate);
			myWhisperQue.offer(TalkFactory.attackRemark(attackingTarget));
		}
	}

	@Override
	public Agent attackAction15() {
		if (attackingTarget != null) {
			return attackingTarget;
		} else if (!attackingCandidate.isEmpty()) {
			return RandomSelect.getOP(gameResult.getWinMap(), attackingCandidate);
		} else {
			return RandomSelect.getOP(gameResult.getWinMap(), getAliveOthersList());
		}
	}
}
