package com.gmail.romanesco2090.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 便利ツールを使えるクラス
 * @author k15096kk
 *
 */
public class Gadget {

	/**
	 * リストに重複していない要素のみ追加する
	 * @param useList リスト
	 * @param elements 追加する要素(重複可能)
	 */
	@SafeVarargs
	public static <T> void addList(List<T> useList, T... elements) {
		for (T element : elements) {
			if (!useList.contains(element)) {
				useList.add(element);
			}
		}
	}

	/**
	 * リストに追加リストの中から重複していない要素のみを追加する
	 * @param useList リスト
	 * @param addLists 追加するリスト(重複可能)
	 * @param <T>
	 */
	@SafeVarargs
	public static <T> void addAllList(List<T> useList, List<T>... addLists) {
		for (List<T> addList : addLists) {
			for (T e : addList) {
				addList(useList, e);
			}
		}
	}

	/**
	 * 指定したリストから複数のリスト内容を排除する
	 * @param useList 指定するリスト
	 * @param removeLists  排除するリスト(重複可能)
	 */
	@SafeVarargs
	public static <T> void removeAll(List<T> useList, List<T>... removeLists) {
		for (List<T> rl : removeLists) {
			useList.removeAll(rl);
		}
	}

	/**
	 * 指定したリストから複数のセット内容を排除する
	 * @param useList 指定するリスト
	 * @param removeSets  排除するセット(重複可能)
	 */
	@SafeVarargs
	public static <T> void removeAll(List<T> useList, Set<T>... removeSets) {
		for (Set<T> rl : removeSets) {
			useList.removeAll(rl);
		}
	}

	/**
	 * 指定したリストから複数の要素を排除する
	 * @param useList 指定するリスト
	 * @param elements 排除する要素(重複可能)
	 */
	@SafeVarargs
	public static <T> void removeAll(List<T> useList, T... elements) {
		for (T e : elements) {
			useList.remove(e);
		}
	}


	/**
	 * 指定したリストから複数のリスト内容を排除したリストを返す
	 * @param useList 指定するリスト
	 * @param removeLists 排除するリスト(重複可能)
	 * @return 排除したリスト
	 */
	@SafeVarargs
	public static <T> List<T> remainList(List<T> useList, List<T>... removeLists){
		List<T> tmpList = new ArrayList<>(useList);
		removeAll(tmpList, removeLists);
		return tmpList;
	}

	/**
	 * 指定したリストから複数のリスト内容を排除したリストを返す
	 * @param useList 指定するリスト
	 * @param removeLists 排除するセット(重複可能)
	 * @return 排除したリスト
	 */
	@SafeVarargs
	public static <T> List<T> remainList(List<T> useList, Set<T>... removeSets){
		List<T> tmpList = new ArrayList<>(useList);
		removeAll(tmpList, removeSets);
		return tmpList;
	}

	/**
	 * 指定したリストから複数の要素を排除したリストを返す
	 * @param useList 指定するリスト
	 * @param removeLists 排除する要素(重複可能)
	 * @return 排除したリスト
	 */
	@SafeVarargs
	public static <T> List<T> remainList(List<T> useList, T... elements){
		List<T> tmpList = new ArrayList<>(useList);
		removeAll(tmpList, elements);
		return tmpList;
	}
}
