package com.gmail.romanesco2090.util;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.aiwolf.common.data.Agent;

public class RandomSelect {

	/**
	 * 対象リストからランダムに要素を取得
	 * @param targetList 対象リスト
	 * @return ランダムに選ばれた要素
	 */
	public static <T> T get(List<T> targetList) {
		if(targetList.isEmpty()) {
			return null;
		}
		return targetList.get(getNum(targetList.size()));
	}

	/**
	 * 対象リストの最も勝利数の多いエージェントからランダムにエージェントを取得
	 * @param winMap エージェントごとの勝利数マップ
	 * @param targetList 対象リスト
	 * @return 勝利数が最も多いエージェントからランダムに選ばれたエージェント
	 */
	public static Agent getOP(Map<Agent, Integer> winMap,List<Agent> targetList) {
		int maxWin = 0;
		List<Agent> maxWinAgentList = new LinkedList<>();
		if(targetList.isEmpty()) {
			return null;
		}
		for(Agent agent : targetList) {
			if(maxWin < winMap.get(agent)) {
				maxWinAgentList.clear();
				maxWinAgentList.add(agent);
			} else if(maxWin == winMap.get(agent)) {
				maxWinAgentList.add(agent);
			}
		}
		return RandomSelect.get(maxWinAgentList);
	}

	/**
	 * 対象リストの最も勝利数の少ないエージェントからランダムにエージェントを取得
	 * @param winMap エージェントごとの勝利数マップ
	 * @param targetList 対象リスト
	 * @return 勝利数が最も少ないエージェントからランダムに選ばれたエージェント
	 */
	public static Agent getDust(Map<Agent, Integer> winMap,List<Agent> targetList) {
		int minWin = 100;
		List<Agent> minWinAgentList = new LinkedList<>();
		if(targetList.isEmpty()) {
			return null;
		}
		for(Agent agent : targetList) {
			if(minWin > winMap.get(agent)) {
				minWinAgentList.clear();
				minWinAgentList.add(agent);
			} else if(minWin == winMap.get(agent)) {
				minWinAgentList.add(agent);
			}
		}
		return RandomSelect.get(minWinAgentList);
	}

	/**
	 * 0~(乱数設定値-1)までのint型乱数作成
	 * @param value 乱数設定値
	 * @return int型乱数
	 */
	public static int getNum(int value) {
		int randomValue = 0;
		randomValue = new Random().nextInt(value);
		return randomValue;
	}
}
